package ;

	import org.cove.ape.APEngine;import org.cove.ape.AbstractCollection;import org.cove.ape.AbstractConstraint;import org.cove.ape.AbstractItem;import org.cove.ape.AbstractParticle;import org.cove.ape.CircleParticle;import org.cove.ape.Collision;import org.cove.ape.CollisionDetector;import org.cove.ape.CollisionResolver;import org.cove.ape.Composite;import org.cove.ape.Group;import org.cove.ape.Interval;import org.cove.ape.MathUtil;import org.cove.ape.RectangleParticle;import org.cove.ape.RimParticle;import org.cove.ape.SpringConstraint;import org.cove.ape.SpringConstraintParticle;import org.cove.ape.Vector;import org.cove.ape.WheelParticle;

	class Robot extends Group {
		public var px(get_px,null):Float;
		public var py(get_py,null):Float;
		public var stiffness(null,set_stiffness):Float;


		private var body:Body;
		private var motor:Motor;

		private var direction:Int;
		private var powerLevel:Float;

		private var powered:Bool;
		private var legsVisible:Bool;

		private var legLA:Leg;
		private var legRA:Leg;
		private var legLB:Leg;
		private var legRB:Leg;
		private var legLC:Leg;
		private var legRC:Leg;

		public function new(px:Float, py:Float, scale:Float, power:Float) {
			super();

			// legs
			legLA = new Leg(px, py, -1, scale, 2, 0x444444, 1, 0x222222, 1);
			legRA = new Leg(px, py,  1, scale, 2, 0x444444, 1, 0x222222, 1);
			legLB = new Leg(px, py, -1, scale, 2, 0x666666, 1, 0x444444, 1);
			legRB = new Leg(px, py,  1, scale, 2, 0x666666, 1, 0x444444, 1);
			legLC = new Leg(px, py, -1, scale, 2, 0x888888, 1, 0x666666, 1);
			legRC = new Leg(px, py,  1, scale, 2, 0x888888, 1, 0x666666, 1);

			// body
			body = new Body(legLA.fix, legRA.fix, Std.int(30 * scale), 2, 0x336699, 1);

			// motor
			motor = new Motor(body.center, 8 * scale, 0x336699);

			// connect the body to the legs
			var connLA:SpringConstraint = cast( new SpringConstraint(body.left,  legLA.fix, 1),SpringConstraint);
			var connRA:SpringConstraint = cast( new SpringConstraint(body.right, legRA.fix, 1),SpringConstraint);
			var connLB:SpringConstraint = cast( new SpringConstraint(body.left,  legLB.fix, 1),SpringConstraint);
			var connRB:SpringConstraint = cast( new SpringConstraint(body.right, legRB.fix, 1),SpringConstraint);
			var connLC:SpringConstraint = cast( new SpringConstraint(body.left,  legLC.fix, 1),SpringConstraint);
			var connRC:SpringConstraint = cast( new SpringConstraint(body.right, legRC.fix, 1),SpringConstraint);


			// connect the legs to the motor
			legLA.cam.position = motor.rimA.position;
			legRA.cam.position = motor.rimA.position;
			var connLAA:SpringConstraint = cast( new SpringConstraint(legLA.cam, motor.rimA, 1),SpringConstraint);
			var connRAA:SpringConstraint = cast( new SpringConstraint(legRA.cam, motor.rimA, 1),SpringConstraint);

			legLB.cam.position = motor.rimB.position;
			legRB.cam.position = motor.rimB.position;
			var connLBB:SpringConstraint = cast( new SpringConstraint(legLB.cam, motor.rimB, 1),SpringConstraint);
			var connRBB:SpringConstraint = cast( new SpringConstraint(legRB.cam, motor.rimB, 1),SpringConstraint);

			legLC.cam.position = motor.rimC.position;
			legRC.cam.position = motor.rimC.position;
			var connLCC:SpringConstraint = cast( new SpringConstraint(legLC.cam, motor.rimC, 1),SpringConstraint);
			var connRCC:SpringConstraint = cast( new SpringConstraint(legRC.cam, motor.rimC, 1),SpringConstraint);

			connLAA.setLine(2,0x999999);
			connRAA.setLine(2,0x999999);
			connLBB.setLine(2,0x999999);
			connRBB.setLine(2,0x999999);
			connLCC.setLine(2,0x999999);
			connRCC.setLine(2,0x999999);

			// add to the engine
			addComposite(legLA);
			addComposite(legRA);
			addComposite(legLB);
			addComposite(legRB);
			addComposite(legLC);
			addComposite(legRC);

			addComposite(body);
			addComposite(motor);

			addConstraint(connLA);
			addConstraint(connRA);
			addConstraint(connLB);
			addConstraint(connRB);
			addConstraint(connLC);
			addConstraint(connRC);

			addConstraint(connLAA);
			addConstraint(connRAA);
			addConstraint(connLBB);
			addConstraint(connRBB);
			addConstraint(connLCC);
			addConstraint(connRCC);

			direction = -1;
			powerLevel = power;

			powered = true;
			legsVisible = true;
		}


		public function get_px():Float {
			return body.center.px;
		}


		public function get_py():Float {
			return body.center.py;
		}


		public function run():Void {
			motor.run();
		}


		public function togglePower():Void {

			powered = !powered;

			if (powered) {
				motor.power = powerLevel * direction;
				stiffness = 1;
				APEngine.damping = 0.99;
			} else {
				motor.power = 0;
				stiffness = 0.2;
				APEngine.damping = 0.35;
			}
		}


		public function toggleDirection():Void {
			direction *= -1;
			motor.power = powerLevel * direction;
		}


		public function toggleLegs():Void {
			legsVisible = ! legsVisible;
			if (!legsVisible) {
				legLA.visible = false;
				legRA.visible = false;
				legLB.visible = false;
				legRB.visible = false;
			} else {
				legLA.visible = true;
				legRA.visible = true;
				legLB.visible = true;
				legRB.visible = true;
			}
		}


		public function set_stiffness(s:Float) {

			// top level constraints in the group
			var i:Int = 0;
			while( i < constraints.length) {

				var sp:SpringConstraint = cast( constraints[i],SpringConstraint);
				sp.stiffness = s;
				 i++;
			}

			// constraints within this groups composites
			var j:Int = 0;
			while( j < composites.length) {

				i = 0;
				while( i < composites[j].constraints.length) {

					var sp = composites[j].constraints[i];
					sp.stiffness = s;
					 i++;
				}
				 j++;
			}
			return s;
		}
	}

